import subprocess
import tempfile

kattis_name = "Java"
file_extension = "java"
emoji = "☕️"
default_content = """class Solution 
{ 
    public static void main(String args[]) 
    { 
        System.out.println("Hello, World"); 
    } 
}
"""
main_class = "Solution"

def run_program(file, input_content):
    # Make temporary path
    dirpath = tempfile.mkdtemp()
    subprocess.call(['javac', file, '-d', dirpath])

    try:
        output = subprocess.check_output(['java', '-cp', dirpath, 'Solution'], input=input_content)
        return output.decode("utf-8")
    except subprocess.CalledProcessError as e:
        return e.output.decode("utf-8") # We still want the output when programs exits with an error


import subprocess
import tempfile

kattis_name = "Rust"
file_extension = "rs"
emoji = "⚙️"
default_content = """
use std::io;
mod scanner;

use scanner::Scanner;

fn main() {
    let (stdin, stdout) = (io::stdin(), io::stdout());
    let mut scan = Scanner::new(stdin.lock());
    let x = scan.token::<i32>();
    println!("{}", x);
}
"""
main_class = "solution.rs"
additional_files = {
    "scanner.rs": """
pub struct UnsafeScanner<R> {
    reader: R,
    buf_str: Vec<u8>,
    buf_iter: str::SplitAsciiWhitespace<'static>,
}

impl<R: io::BufRead> UnsafeScanner<R> {
    pub fn new(reader: R) -> Self {
        Self {
            reader,
            buf_str: vec![],
            buf_iter: "".split_ascii_whitespace(),
        }
    }

    /// This function should be marked unsafe, but noone has time for that in a
    /// programming contest. Use at your own risk!
    pub fn token<T: str::FromStr>(&mut self) -> T {
        loop {
            if let Some(token) = self.buf_iter.next() {
                return token.parse().ok().expect("Failed parse");
            }
            self.buf_str.clear();
            self.reader
                .read_until(b'\\n', &mut self.buf_str)
                .expect("Failed read");
            self.buf_iter = unsafe {
                let slice = str::from_utf8_unchecked(&self.buf_str);
                std::mem::transmute(slice.split_ascii_whitespace())
            }
        }
    }
}

use std::io;
use std::str;

/// Reads white-space separated tokens one at a time.
pub struct Scanner<R> {
    reader: R,
    buffer: Vec<String>,
}

impl<R: io::BufRead> Scanner<R> {
    pub fn new(reader: R) -> Self {
        Self {
            reader,
            buffer: vec![],
        }
    }

    /// Use "turbofish" syntax token::<T>() to select data type of next token.
    ///
    /// # Panics
    ///
    /// Panics if there's an I/O error or if the token cannot be parsed as T.
    pub fn token<T: str::FromStr>(&mut self) -> T {
        loop {
            if let Some(token) = self.buffer.pop() {
                return token.parse().ok().expect("Failed parse");
            }
            let mut input = String::new();
            self.reader.read_line(&mut input).expect("Failed read");
            self.buffer = input.split_whitespace().rev().map(String::from).collect();
        }
    }
}
    """
}

additional_files_that_shouldnt_be_submitted = {
    "Cargo.toml": """[package]
name = "kattis-problem"
version = "0.1.0"

[lib]
name = "solution"
path = "solution.rs"
"""
}


def run_program(file, input_content):
    # Make temporary path
    dirpath = tempfile.mkdtemp()
    subprocess.call(["rustc", file, "-o", dirpath + "/main", "-A", "warnings"])

    try:
        output = subprocess.check_output([dirpath + "/main"], input=input_content)
        return output.decode("utf-8")
    except subprocess.CalledProcessError as e:
        return e.output.decode(
            "utf-8"
        )  # We still want the output when programs exits with an error



import glob
import re
import pkgutil
import kattiskitten.languages as languages

SUPPORTED_LANGUAGES = []
LANGUAGE_EXTENSIONS = {}
CONFIGS = {}

for importer, language, ispkg in pkgutil.iter_modules(languages.__path__):
    SUPPORTED_LANGUAGES.append(language)
    config = importer.find_module(language).load_module(language)

    LANGUAGE_EXTENSIONS[config.file_extension] = language
    CONFIGS[language] = config

def get_config(language):
    if language not in CONFIGS:
        raise ValueError(f"Language not supported. Supported languages are: {', '.join(SUPPORTED_LANGUAGES)}")
    return CONFIGS[language]



def determine_language(problem):
    solution = glob.glob(f"./{problem}/solution.*")

    if len(solution) < 1:
        raise ValueError("Couldn't find any program matching patten (solution.*)")
    if len(solution) > 1:
        raise ValueError(
            "Found more than one program matching patten (solution.*). It currently only supports one")

    m = re.search(r".*\.(.+?)$", solution[0])

    if m:
        extension = m.group(1)
        language = LANGUAGE_EXTENSIONS[extension]
        if not language:
            raise ValueError(
                f"Couldn't find supported language with extension {extension}")

        return language



import requests
import configparser
import os
from pathlib import Path
import re
import webbrowser
import sys
import kattiskitten.language_detector as language_detector
import kattiskitten.config as cfg

_HEADERS = {'User-Agent': 'kattis-cli-submit'}

def get_active_org():
    return cfg.get('active_org') or 'open.kattis.com'

def get_url(path):
    return 'https://%s/%s' % (get_active_org(), path)

def login(login_url, username, token=None):
    login_args = {'user': username, 'script': 'true', 'token': token}
    return requests.post(login_url, data=login_args, headers=_HEADERS)

def login_from_config():
    org = get_active_org()

    username = cfg.get('username', org)
    token = cfg.get('token', org)
    loginurl = get_url('login')

    return login(loginurl, username, token)

def open_submission(submit_response):
    submissions_url = get_url('submissions')

    m = re.search(r'Submission ID: (\d+)', submit_response)
    if m:
        submission_id = m.group(1)
        url = '%s/%s' % (submissions_url, submission_id)
        webbrowser.open(url)
            

def submit_problem(problem, sub_files, open=True):
    lang = language_detector.determine_language(problem)
    lang_config = language_detector.get_config(lang)

    login_reply = login_from_config()
    data = {'submit': 'true',
            'submit_ctr': 2,
            'language': lang_config.kattis_name,
            'mainclass': lang_config.main_class,
            'problem': problem,
            'tag': 'Submitted with kattis kitten 🦄',
            'script': 'true'}

    result = requests.post(get_url('submit'), data=data, files=sub_files, cookies=login_reply.cookies, headers=_HEADERS)

    plain_result = result.content.decode('utf-8').replace('<br />', '\n')
    if len(plain_result) < 1000:
        print(plain_result)
        if open: open_submission(plain_result)
    else:
        print("Authentication failed :/")
        print("Reauthenticate with \"kk org add\"")


import glob
import subprocess
import colorful as cf
import kattiskitten.language_detector as language_detector
import time

def test_problem(problem, log=True):
    if log: print(f"👷‍ Testing {problem}...")

    lang = language_detector.determine_language(problem)
    lang_config = language_detector.get_config(lang)
    if log: print(f"👷‍ Language = {lang_config.kattis_name} {lang_config.emoji}\n")

    inputs = glob.glob(f"./{problem}/*.in")

    count = 0
    failed = False
    for input in inputs:
        count += 1
        if log:
            print(f"🔎 Test number {count}:")

        input_file = open(input, "rb")
        input_content = input_file.read()

        program_path = f"./{problem}/solution.{lang_config.file_extension}"
        start = time.time()
        output_string = lang_config.run_program(program_path, input_content)
        end = time.time()
        answer = input[:-3] + ".ans"
        answer_file = open(answer, "r")
        answer_content = answer_file.read()

        if output_string.replace("\r\n", "\n") != answer_content.replace("\r\n", "\n"):
            if log:
                print(cf.bold_red("❌ Failed..."))
                print("__________INPUT____________")
                print(input_content.decode('utf-8'))
                print("__________INPUT____________")
                print(cf.red("__________OUTPUT___________"))
                print(cf.red(output_string))
                print(cf.red("__________OUTPUT___________"))
                print("__________EXPECTED_________")
                print(answer_content)
                print("__________EXPECTED_________")

            failed = True
        elif log:
            print(cf.bold_green(f"✅ Test succesful! (completed in {end-start} seconds)\n"))



    return not failed


import configparser
import os
from pathlib import Path

config_path = os.path.join(Path.home(), '.kattiskitten', 'config.ini')

cfg = configparser.ConfigParser()
cfg.read(config_path)


def get(key, section='DEFAULT'):
    return cfg[section][key] if key in cfg[section] else None

def set(key, value, section='DEFAULT'):
    if section not in cfg:
        cfg[section] = {}
    cfg[section][key] = value

    if not os.path.exists(os.path.dirname(config_path)):
        os.makedirs(os.path.dirname(config_path))

    with open(config_path, 'w') as file:
        cfg.write(file)

def sectionExists(section):
    return True if section in cfg else False


__version__ = "0.1.0"

import configparser
import glob
import os
import re
import subprocess
import sys
import tempfile
import webbrowser
import zipfile

import click
import colorful as cf
import requests

import kattiskitten.config as cfg
import kattiskitten.language_detector
from kattiskitten.submitter import submit_problem
from kattiskitten.tester import test_problem

__author__ = "Felix Qvist"


@click.group()
def main():
    """
    Simple CLI for downloading and testing kattis problems
    """
    print(f"Using {cfg.get('active_org') or 'open.kattis.com'}")
    pass


@main.command()
@click.argument("problem")
def test(problem):
    """This tests a kattis problem using provided test problems"""
    if problem == "all":
        problems = glob.glob(f"./*/")

        completed_count = 0
        total_score = 0
        earned_score = 0
        for p in problems:
            p = re.sub(r"[^\w]", "", p)
            res = test_problem(p, False)
            total_score += score
            if res:
                print(f"✅ {p}")
                completed_count += 1
                earned_score += score
            else:
                print(f"❌ {p}")

        print(f"\n Completed {completed_count}/{len(problems)}")

    else:
        res = test_problem(problem)
        if res:
            print(f"Woop woop 🎉")


@main.command()
@click.option("--language", default="python3")
@click.argument("problem")
def get(problem, language):
    """This command downloads a kattis problem and test files"""
    hostname = (
        cfg.get("active_org")
        if cfg.get("active_org") is not None
        else "open.kattis.com"
    )
    res = requests.get(
        f"https://{hostname}/problems/{problem}/file/statement/samples.zip"
    )

    language_config = language_detector.get_config(language)

    if res.status_code == 404:
        print(
            f"Couldn't find problem '{problem}'. Maybe you typed it wrong? Or no test files exist"
        )
    else:
        print("Downloading samples")
        tmp_file = tempfile.NamedTemporaryFile(delete=False)
        open(tmp_file.name, "wb").write(res.content)

        with zipfile.ZipFile(tmp_file.name, "r") as zip_ref:
            zip_ref.extractall(f"./{problem}")
            print(f"Samples downloaded to './{problem}'")
        tmp_file.close()
        os.unlink(tmp_file.name)

        # Create main.xx with template
        if not os.path.exists(f"./{problem}/solution.{language_config.file_extension}"):
            open(f"./{problem}/solution.{language_config.file_extension}", "w").write(
                language_config.default_content
            )

        if "additional_files" in dir(language_config):
            for file in language_config.additional_files:
                if not os.path.exists(f"./{problem}/{file}"):
                    open(f"./{problem}/{file}", "w").write(
                        language_config.additional_files[file]
                    )

        if "additional_files_that_shouldnt_be_submitted" in dir(language_config):
            for file in language_config.additional_files_that_shouldnt_be_submitted:
                if not os.path.exists(f"./{problem}/{file}"):
                    open(f"./{problem}/{file}", "w").write(
                        language_config.additional_files_that_shouldnt_be_submitted[
                            file
                        ]
                    )


@main.command()
@click.argument("problem")
def submit(problem):
    """This command submits a problem to kattis"""
    lang = language_detector.determine_language(problem)
    lang_config = language_detector.get_config(lang)
    f = f"./{problem}/solution.{lang_config.file_extension}"

    sub_files = []
    with open(f) as sub_file:
        sub_files.append(
            (
                "sub_file[]",
                (os.path.basename(f), sub_file.read(), "application/octet-stream"),
            )
        )

    if "additional_files" in dir(lang_config):
        for file_name in lang_config.additional_files:
            f = f"./{problem}/{file_name}"
            with open(f) as sub_file:
                sub_files.append(
                    (
                        "sub_file[]",
                        (
                            os.path.basename(f),
                            sub_file.read(),
                            "application/octet-stream",
                        ),
                    )
                )

    submit_problem(problem, sub_files)


@main.group()
def org():
    """Change organization settings"""
    pass


@org.command()
@click.argument("organization", required=False)
def current(organization):
    """Gets or sets the organization"""
    if not organization:
        print("Current org is", cfg.get("active_org") or "open.kattis.com")
    elif cfg.sectionExists(organization) or organization == "open.kattis.com":
        cfg.set("active_org", organization)
        print(f"Changed active org to {organization}")
    else:
        print("You haven't added this organization. Add it with\nkk org add")


@org.command()
def add():
    """Add a new organization"""
    hostname = click.prompt("Organization url", type=str, default="open.kattis.com")
    res = requests.get(f"https://{hostname}")
    if res.status_code == 404:
        print(f"Couldn't find org ({res.request.url}). Did you spell it correctly?")
        return

    print("\n\nNow you need to download the configuration file from the following url:")
    print(f"https://{hostname}/download/kattisrc\n\n")

    username = click.prompt("Kattis username (copy from .kattisrc)", type=str)

    res = requests.get(f"https://{hostname}/users/{username}")
    if res.status_code == 404:
        print(f"Couldn't find user ({res.request.url}). Did you spell it correctly?")
        return

    token = click.prompt("Kattis token (copy from .kattisrc)", type=str)

    cfg.set("username", username, hostname)
    cfg.set("token", token, hostname)


if __name__ == "__main__":
    main()



import subprocess

kattis_name = "Python 3"
file_extension = "py"
default_content = 'print("Hello world!")'
emoji = "🐍"
main_class = "solution.py"

def run_program(file, input_content):
    try:
        output = subprocess.check_output(['python3', file], input=input_content)
        return output.decode("utf-8")
    except subprocess.CalledProcessError as e:
        return e.output.decode("utf-8") # We still want the output when programs exits with an error


# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['kattiskitten', 'kattiskitten.languages']

package_data = \
{'': ['*']}

install_requires = \
['click==7.0', 'colorful==0.5.4', 'requests==2.22.0']

entry_points = \
{'console_scripts': ['kk = kattiskitten:main']}

setup_kwargs = {
    'name': 'kattiskitten',
    'version': '0.14.0',
    'description': 'Kattis CLI - Easily download, test and submit kattis problems',
    'long_description': "# Kattis kitten\n\n![Repo size](https://img.shields.io/github/repo-size/FelixDQ/kattis-kitten)\n[![PyPI version](https://img.shields.io/pypi/v/kattiskitten)](https://pypi.org/project/kattiskitten/)\n\nKattis CLI - Easily download, test and submit kattis problems\n\n```\nUsage: kk [OPTIONS] COMMAND [ARGS]...\n\n  Simple CLI for downloading and testing kattis problems\n\nOptions:\n  --help  Show this message and exit.\n\nCommands:\n  get       This command downloads a kattis problem and test files\n  problems  Simply opens https://open.kattis.com/problems in your webbrowser\n  submit    This command submits a problem to kattis\n  test      This tests a kattis problem using provided test problems\n```\n\nInstallation (requires python >= 3.6):\n\n```\npip3 install kattiskitten\n```\n\n# Commands\n\nDownload test files.\n\n```\n> kk get rationalsequence\nDownloading samples\nSamples downloaded to './rationalsequence'\n```\n\nTest the problem\n\n```\n> kk test rationalsequence\n👷\u200d Testing rationalsequence...\n👷\u200d Language = Python 3 🐍\n\n🔎 Test number 1:\n❌ Failed...\n__________INPUT____________\n5\n1 1/1\n2 1/3\n3 5/2\n4 2178309/1346269\n5 1/10000000\n\n__________INPUT____________\n__________OUTPUT___________\nHello world!\n\n__________OUTPUT___________\n__________EXPECTED_________\n1 1/2\n2 3/2\n3 2/5\n4 1346269/1860498\n5 10000000/9999999\n\n__________EXPECTED_________\n```\n\nSubmit solution to kattis\n\n```\n> kk submit rationalsequence\nSubmission received. Submission ID: 5030066.\n* Opens web browser on submission page *\n```\n\n# Choose language\n\nThe default language is python3. To change language you can use the `--language` flag on the get command.\n\n```\n> kk get rationalsequence --language java\nDownloading samples\nSamples downloaded to './rationalsequence'\n```\n\nThe other commands will auto detect which language you have chosen.\n\n# Supported languages\n\n- Python3\n- Java\n- C++\n- C\n- Rust\n- Contribute by adding [more languages](https://github.com/FelixDQ/kattis-kitten/tree/master/kattiskitten/languages)! :-)\n",
    'author': 'Felix Qvist',
    'author_email': 'felix@qvist.dev',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/FelixDQ/kattis-kitten',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.6,<4.0',
}


setup(**setup_kwargs)



import subprocess
import tempfile
import os

kattis_name = "C"
file_extension = "c"
emoji = "👻"
default_content = """#include <stdio.h>
int main() {
   printf("Hello, World!");
   return 0;
}
"""
main_class = "solution.c"

def run_program(file, input_content):
    # Make temporary path
    dirpath = tempfile.mkdtemp()
    compiled_path = os.path.join(dirpath, 'program')
    subprocess.call(['gcc', file, '-o', compiled_path])

    try:
        output = subprocess.check_output([compiled_path], input=input_content)
        return output.decode("utf-8")
    except subprocess.CalledProcessError as e:
        return e.output.decode("utf-8") # We still want the output when programs exits with an error